"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.isMonotonic = void 0;
const is_monotonic_options_1 = require("./is-monotonic.options");
function compareValues(a, b, monotonicity) {
    switch (monotonicity) {
        case is_monotonic_options_1.Monotonicity.WEAKLY_INCREASING:
            return b >= a;
        case is_monotonic_options_1.Monotonicity.STRICTLY_INCREASING:
            return b > a;
        case is_monotonic_options_1.Monotonicity.WEAKLY_DECREASING:
            return b <= a;
        case is_monotonic_options_1.Monotonicity.STRICTLY_DECREASING:
            return b < a;
        default:
            throw new Error('Unexpected monotonicity value');
    }
}
const allowedValues = new Set(Object.values(is_monotonic_options_1.Monotonicity));
function isMonotonic(values, selector, monotonicity) {
    if (!allowedValues.has(monotonicity)) {
        const allowedValuesString = [...allowedValues].map(value => `"${value}"`).join(', ');
        throw new TypeError(`Unknown monotonicity type "${monotonicity}" (expected one of ${allowedValuesString})`);
    }
    return (Array.isArray(values) &&
        values.reduce((satisfied, item, index) => satisfied && (index === 0 || compareValues(selector(values[index - 1]), selector(item), monotonicity)), true));
}
exports.isMonotonic = isMonotonic;
//# sourceMappingURL=is-monotonic.predicate.js.map